package cn.ibizlab.core.extensions.service;

import cn.ibizlab.core.lite.extensions.domain.Setting;
import cn.ibizlab.core.lite.service.IDstDataSourceService;
import cn.ibizlab.core.lite.service.impl.DstDataSourceServiceImpl;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.dynamic.datasource.DynamicRoutingDataSource;
import com.baomidou.dynamic.datasource.creator.DruidDataSourceCreator;
import com.baomidou.dynamic.datasource.spring.boot.autoconfigure.DataSourceProperty;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.slf4j.Slf4j;
import cn.ibizlab.core.lite.domain.DstDataSource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Primary;
import org.springframework.util.StringUtils;

import javax.sql.DataSource;
import java.util.*;

/**
 * 实体[数据源] 自定义服务对象
 */
@Slf4j
@Primary
@Service("DstDataSourceExService")
public class DstDataSourceExService extends DstDataSourceServiceImpl {

    @Override
    protected Class currentModelClass() {
        return com.baomidou.mybatisplus.core.toolkit.ReflectionKit.getSuperClassGenericType(this.getClass().getSuperclass(), 1);
    }

    /**
     * [BuildDS:buildDS] 行为扩展
     * @param dstDataSource
     * @return
     */
    @Override
    @Transactional
    public DstDataSource buildDS(DstDataSource dstDataSource) {

        if(!(StringUtils.isEmpty(dstDataSource.getDsCfg())))
        {
            DataSourceProperty dataSourceProperty = new DataSourceProperty();
            DynamicRoutingDataSource ds =  dataSource;
            List<Setting> settings= JSONArray.parseArray(dstDataSource.getDsCfg(),Setting.class);
            for(Setting setting:settings)
            {
                if(setting.getProperty().equalsIgnoreCase("driver-class-name"))
                    dataSourceProperty.setDriverClassName(setting.getValue());
                else if(setting.getProperty().equalsIgnoreCase("username"))
                    dataSourceProperty.setUsername(setting.getValue());
                else if(setting.getProperty().equalsIgnoreCase("password"))
                    dataSourceProperty.setPassword(setting.getValue());
                else if(setting.getProperty().equalsIgnoreCase("url"))
                    dataSourceProperty.setUrl(setting.getValue());
                else if(setting.getProperty().equalsIgnoreCase("defaultSchema"))
                    dataSourceProperty.setSchema(setting.getValue());
            }
            DataSource dataSource = druidDataSourceCreator.createDataSource(dataSourceProperty);

            ds.addDataSource(dstDataSource.getDsName(), dataSource);
        }

        return super.buildDS(dstDataSource);
    }

    @Autowired
    @Lazy
    private DynamicRoutingDataSource dataSource;

    @Autowired
    @Lazy
    private DruidDataSourceCreator druidDataSourceCreator;


    public void initDataSource(String dsName)
    {
       // if(dataSource.getDataSource(dsName)==null)
        {
            DstDataSource dstDataSource=this.getOne(Wrappers.<DstDataSource>lambdaQuery().eq(DstDataSource::getDsId,dsName).or().eq(DstDataSource::getDsName,dsName),false);
            if(dstDataSource!=null)
            {
                dstDataSource.setDsName(dsName);
                buildDS(dstDataSource);
            }
        }
    }

    @Override
    public boolean update(DstDataSource et) {
        if(!super.update(et))
            return false;
        DynamicRoutingDataSource ds = (DynamicRoutingDataSource) dataSource;
        if(ds.getDataSource(et.getDsId())!=null)
            ds.removeDataSource(et.getDsId());
        if(ds.getDataSource(et.getDsName())!=null)
            ds.removeDataSource(et.getDsName());
        buildDS(et);
        return true;
    }
}

